"""
Copyright (c) 2022 Center for Astrophysics | Harvard & Smithsonian

This software is licensed for use as described in the LICENSE file in
the root directory of this distribution.

Manage things to do with arrays

Originator: Aaron Oppenheimer March 2020
"""
import csv
import numpy as np
import ehtim as eh
from pathlib import Path
from .station import Station

_THE_ARRAYS = None

def _init_arrays():
    """ do the initial setup on arrays """
    global _THE_ARRAYS
    if _THE_ARRAYS is None:

        # set up the arrays, which are just lists of station codes
        _THE_ARRAYS = {}
        path=str(Path(__file__).parent) + '/config'
        with open(f'{path}/arrays.csv', newline='') as csvfile:
            reader = csv.reader(csvfile, delimiter=',')
            for row in reader:
                _THE_ARRAYS[row[0]] = [x for x in row[1:] if x]
_init_arrays()

class Array:
    """
    Class to represent an Array, comprising a set of Station objects. The module loads a set
    of known arrays that can be accessed through class methods.
    """
    name = None
    _stations = []

    @staticmethod
    def get_list():
        """ Get the list of arrays in the database as a list of names """
        return list(_THE_ARRAYS.keys())

    @classmethod
    def get_default_array_name(cls):
        """ Returns the name of the first known array in the builtin database """
        return cls.get_list()[0]

    @classmethod
    def get_default(cls):
        """ Returns an Array object representing the default array """
        return cls.from_name(cls.get_default_array_name())

    @classmethod
    def get_station_names(cls, name):
        """ Get list of station names associated with an array """
        return _THE_ARRAYS[name]

    @classmethod
    def from_name(cls, name):
        """
        Returns an Array object from the database by name. Will raise an exception if the
        array name is unknown.
        """
        stations = [Station.from_name(x) for x in _THE_ARRAYS[name]]
        return cls(name, stations)

    def __init__(self, name, stations):
        """Initialize an Array object"""
        self.name = name if name else '[none]'
        self.stations(stations)

    def stations(self, stns = None):
        """ return the stations comprising this array, or set it """
        if stns:
            if not type(stns) is list:
                raise ValueError("Can only add lists of Stations to an array")
            if not sum([1 if type(x) is Station else 0 for x in stns]) == len(stns):
                raise ValueError("Can only add lists of Stations to an array")
            names = [s.name for s in stns]
            for i,n in enumerate(names[:-1]):
                if n in names[i+1:]:
                    raise ValueError("Can't have stations with duplicate names in an array")

            self._stations = stns

        return self._stations

    def to_ehtim_array(self, freq, filled=0.7, month=5):
        """
        Returns an ehtim array object.
         
         freq is the measurement frequency (GHz)
         filled is the geometric filling factor (unobscured telescope fraction)
         month is the observation month (1-12)
        
        """
        tarr = np.recarray(len(self.stations()),dtype=eh.const_def.DTARR)
        for isite, site in enumerate(self.stations()):
            xyz = site.xyz()
            SEFD = site.SEFD(freq,site.elevation,filled=filled,month=month)
            tarr[isite]['site'] = site.name
            tarr[isite]['x'] = xyz[0]
            tarr[isite]['y'] = xyz[1]
            tarr[isite]['z'] = xyz[2]
            tarr[isite]['sefdr'] = SEFD
            tarr[isite]['sefdl'] = SEFD
        return eh.array.Array(tarr)

    def __str__(self):
        return f'Array {self.name}'

    def __repr__(self):
        return f'Array {self.name}'



"""
Copyright (c) 2022 Center for Astrophysics | Harvard & Smithsonian

This software is licensed for use as described in the LICENSE file in
the root directory of this distribution.

Code to demonstrate use of the cost model

Originator: Aaron Oppenheimer July 2020
"""
from ngehtutil.cost import calculate_costs, CostConfig
from ngehtutil import *

def main():

    config = CostConfig(
        observations_per_year=1,
        days_per_observation=60,
        hours_per_observation=60*8,
        no_upgrade = ['ALMA', 'NOEMA', 'SMA'],
        data_management = 'Cloud'
    )


    site_names = ['OVRO', 'CNI', 'BAJA', 'LAS', 'HAY', 'PIKE', 'BOL', 'CAT', 'BRZ', 'GAM'] + \
        Array.get_station_names("eht2022")
    
    # array = Array.from_name('ngEHT Ref. Array 1.1A + EHT2022')
    array = Array('Phase 1 + 2', [Station.from_name(x) for x in site_names])
    costs, site_costs = calculate_costs(config, array.stations())
    
    print(f'array: {array}')
    print('CONFIGURATION:')
    for x in dir(config):
        if not x[0]=='_':
            print(f'{x}: {getattr(config,x)}')
    for k,v in costs.items():
        print(f'{k},{v}')

    print('\n')

    # print site names, skip first column
    names = ','.join(['SITE'] + list(site_costs.keys()))
    print(names)

    # get the list of things we want to look at for each site
    line_items = site_costs[list(site_costs.keys())[0]].keys()

    for li in line_items:
        print(','.join([li]+[str(site_costs[x][li]) for x in site_costs.keys()]))

    # for k,v in site_costs.items():
    #     print(k)
    #     for sk, sv in v.items():
    #         print(f'{sk},{sv}')


if __name__ == '__main__':
    main()



"""
Copyright (c) 2022 Center for Astrophysics | Harvard & Smithsonian

This software is licensed for use as described in the LICENSE file in
the root directory of this distribution.

Cost model code

Originator: Aaron Oppenheimer March 2020
"""
from .cost_config import CostConfig
from .cost_model import calculate_costs, calculate_capital_costs, calculate_operations_costs, \
    get_cost_constants
from .cost_timing import calculate_costs_over_time



"""
Copyright (c) 2022 Center for Astrophysics | Harvard & Smithsonian

This software is licensed for use as described in the LICENSE file in
the root directory of this distribution.

Originator: Aaron Oppenheimer March 2020
"""
from .target import *
from .source import *
from .cost import *
from .campaign import *
from .station import *
from .array import *
from .program import *



"""
Copyright (c) 2022 Center for Astrophysics | Harvard & Smithsonian

This software is licensed for use as described in the LICENSE file in
the root directory of this distribution.

Code to manage site weather derived from MERRA-2 data

Originator: Aaron Oppenheimer March 2020
"""

import requests
import os
from pathlib import Path
import shutil
import pandas as pd


monthmap = {
    1: '01Jan',
    2: '02Feb',
    3: '03Mar',
    4: '04Apr',
    5: '05May',
    6: '06Jun',
    7: '07Jul',
    8: '08Aug',
    9: '09Sep',
    10: '10Oct',
    11: '11Nov',
    12: '12Dec'
}

urlbase = 'https://raw.githubusercontent.com/Smithsonian/ngeht-weather/main/weather_data'
filenames = [
            'RH.csv',
            'SEFD_info_230.csv',
            'SEFD_info_345.csv',
            'mean_RH.csv',
            'mean_SEFD_info_230.csv',
            'mean_SEFD_info_345.csv',
            'mean_wind_speed.csv',
            'wind_speed.csv',
        ]

homepath=str(Path(__file__).parent) + '/weather_data'

def load_site(sitename, months=None):
    """ Makes sure we have the data for a site for the given months, which is
        a list of month numbers 1-12, or None which means all. """

    if not months:
        months = list(range(1,13))
    else:
        if type(months) is int:
            months=[months]
        if not all(x >= 1 and x <= 12 for x in months):
            raise ValueError('Months must be between 1 and 12')

    fetched = 0

    months = \
        [x for x in months if not os.path.exists(f'{homepath}/{sitename}/{monthmap[x]}/RH.csv')]
    if len(months) == 0:
        return 0 # nothing to load

    # make sure the target directory is here
    if not os.path.isdir(homepath):
        # create the homepath
        os.mkdir(homepath)
    if not os.path.isdir(f'{homepath}/{sitename}'):
        os.mkdir(f'{homepath}/{sitename}')

    for m in months:
        if not os.path.isdir(f'{homepath}/{sitename}/{monthmap[m]}'):
            os.mkdir(f'{homepath}/{sitename}/{monthmap[m]}')
        for f in filenames:
            urlpath = f'{urlbase}/{sitename}/{monthmap[m]}/{f}'
            r = requests.get(urlpath, allow_redirects=True)
            if r.status_code == 404:
                raise ValueError('no such weather data file for {sitename} {monthmap[m]}')
            open(f'{homepath}/{sitename}/{monthmap[m]}/{f}','w').write(r.content.decode("utf-8"))
            fetched = fetched + 1

    return fetched


def delete_sites():
    """ get rid of any downloaded weather files """
    try:
        shutil.rmtree(homepath)
    except FileNotFoundError:
        pass


def get_weather_data(site, type, year, month, day):
    """ return specific information from the weather data """

    # make sure we have the files
    load_site(site, month)

    # get the data
    file = f'{homepath}/{site}/{monthmap[month]}/{type}.csv'
    data = pd.read_csv(file, comment='#')

    ret_data = list((data[(data.loc[:,'year']==year) & \
        (data.loc[:,'day']==day)].iloc[:,3:]).itertuples(name=None, index=False))

    ret = {'index':list(data.columns[3:]),
            'data': ret_data if ret_data else None
            }
    return ret
    # return ret[0] if len(ret)==1 else None if len(ret)==0 else ret



"""
Copyright (c) 2022 Center for Astrophysics | Harvard & Smithsonian

This software is licensed for use as described in the LICENSE file in
the root directory of this distribution.

Code to demonstrate use of the cost model

Originator: Aaron Oppenheimer March 2020
"""
from ngehtutil.cost import *
from ngehtutil import Array, Station
import time


def test():
    config = CostConfig()
    s1 = Station.from_name('KILI')
    s2 = Station.from_name('HAY')
    s3 = Station.from_name('LOS')

    costs1 = calculate_costs(config, [s1])
    costs2 = calculate_costs(config, [s2])
    costs3 = calculate_costs(config, [s3])
    
    print(f'{s1.name} capex:{costs1["TOTAL CAPEX"]}')
    print(f'{s2.name} capex:{costs2["TOTAL CAPEX"]}')
    print(f'{s3.name} capex:{costs3["TOTAL CAPEX"]}')
    

def doit():
    config = CostConfig(
        observations_per_year=1,
        days_per_observation=60,
        hours_per_observation=60*8
    )
    a = Array.from_name('ngEHT Ref. Array 1.1A + EHT2022')
    costs, nscosts = calculate_capital_costs(config, a.stations(), get_cost_constants())

def doit2():
    config = CostConfig(
        observations_per_year=1,
        days_per_observation=60,
        hours_per_observation=60*8
    )
    a = Array.from_name('ngEHT Ref. Array 1.1A + EHT2022')
    t, n = calculate_operations_costs(config, a.stations(), get_cost_constants())

def dishtest():
    config = CostConfig()
    stat = Station.from_name('HAY')
    stat.dishes = None
    for i in range(6,11):
        config.dish_size = i
        costs = calculate_costs(config, [stat])
        print(str(i)+' meters:',costs["TOTAL CAPEX"])
        # for k,v in costs.items():
        #     print(f'{k}: {v}')

def main():
    reps = 1000
    t1 = time.time()
    for i in range(0,reps):
        doit()
    t2 = time.time()
    print(f'time for capital cost calc: {(t2-t1)/reps}')

    t1 = time.time()
    for i in range(0,reps):
        doit2()
    t2 = time.time()
    print(f'time for operations cost calc: {(t2-t1)/reps}')

    test()

if __name__ == '__main__':
    main()
    dishtest()



"""
Copyright (c) 2022 Center for Astrophysics | Harvard & Smithsonian

This software is licensed for use as described in the LICENSE file in
the root directory of this distribution.

Classes to describe VLBI campaigns

Originator: Aaron Oppenheimer March 2020
"""

class Schedule():
    """
    Describes an observation schedule in terms of number of events per year, duration of event in days,
    and data captured by hours observed.
    """
    obs_per_year = None # number of observations in a year
    obs_days = None # number of days for each observation
    obs_hours = None # total number of hours per observation

    def __init__(self, obs_per_year=1, obs_days=5, obs_hours=15):
        """ Initialize a schedule """
        self.obs_per_year = obs_per_year
        self.obs_days = obs_days
        self.obs_hours = obs_hours

    def __repr__(self):
        return f'Schedule({self.obs_per_year}, {self.obs_days}, {self.obs_hours})'

    def __str__(self):
        return f'Schedule: {self.obs_per_year} obs per year; {self.obs_days} days per obs; {self.obs_hours} hours per obs'


    # THIS DOESN'T DO THE RIGHT THING - For 1,2,3 + 1,2,3 we should not end up with 2 4-day events
    # def __add__(self, value):
    #     """ Combine two schedules into a single schedule """

    #     if not type(value) is Schedule:
    #         raise TypeError

    #     return Schedule(obs_per_year = self.obs_per_year + value.obs_per_year,
    #                     obs_days = self.obs_days + value.obs_days,
    #                     obs_hours = self.obs_hours + value.obs_hours)


class Campaign:
    """
    Class to describe a "campaign" which comprises a target in the sky, a source to observe,
    and a schedule for the observation.
    """
    schedule = None
    target = None
    source = None

    def __init__(self, target, source, schedule):
        self.schedule = schedule
        self.target = target
        self.source = source

    def __str__(self):
        return f'{self.source} @ {self.target} for {self.schedule}'

    def __repr__(self):
        return f'{self.source} @ {self.target} for {self.schedule}'


"""
Copyright (c) 2022 Center for Astrophysics | Harvard & Smithsonian

This software is licensed for use as described in the LICENSE file in
the root directory of this distribution.

Testing the program model

Originator: Aaron Oppenheimer March 2020
"""
from ngehtutil import *
import unittest
import copy

class TestClass(unittest.TestCase):
    def test_program_cost(self):
        t = Target.get_default()
        s = Source.get_default()
        sch = Schedule(obs_per_year=1, obs_days=1, obs_hours=12)
        c = Campaign(t,s,sch)
        stn = Station.from_name('OVRO')
        array = Array(f'test', [stn])
        costs, site_costs = Program(array, c).calculate_costs()
        self.assertEqual(type(costs), dict)
        self.assertEqual(type(site_costs), dict)


    def test_program_cost_config_override(self):
        t = Target.get_default()
        s = Source.get_default()
        sch = Schedule(obs_per_year=1, obs_days=1, obs_hours=12)
        c = Campaign(t,s,sch)
        stn = Station.from_name('OVRO')
        array = Array(f'test', [stn])
        prog = Program(array, c)
        costs1, _ = prog.calculate_costs(dish_size=6)
        costs2, _ = prog.calculate_costs(dish_size=10)
        self.assertTrue(costs1['Antenna construction']<costs2['Antenna construction'])
        


"""
Copyright (c) 2022 Center for Astrophysics | Harvard & Smithsonian

This software is licensed for use as described in the LICENSE file in
the root directory of this distribution.

Testing the Station model

Originator: Aaron Oppenheimer March 2020
"""
from ngehtutil import *
from ngehtutil.station_weather import delete_sites
import unittest

class TestClass(unittest.TestCase):
    def test_station_list(self):
        sl1 = Station.get_list()
        self.assertTrue(len(sl1)>0) # should get all stations

        d = Array.get_default_array_name()
        sl2 = Array.get_station_names(d)
        self.assertTrue(len(sl2)<=len(sl1))

        with self.assertRaises(KeyError) as info:
            sl3 = Array.get_station_names('foo')

    def test_station_info(self):
        sl = Station.get_list()
        info = Station(sl[0])
        self.assertEqual(type(info),Station)
        self.assertEqual(info.name, sl[0])

    def test_station_diameter(self):
        s = Station.from_name('CNI') # pick one we know doesn't have a dish
        self.assertEqual(s.dishes, None)

        s.set_diameter(10)
        self.assertEqual(len(s.dishes),1)
        self.assertEqual(s.dishes[0].diameter,10)

    def test_station_SEFD(self):
        sl = Station.get_list()
        s = Station(sl[0])

        s.dishes = None
        with self.assertRaises(ValueError):
            sefd = s.SEFD(230,90)

        s.set_diameter(6)
        sefd = s.SEFD(230,90)
        self.assertTrue(sefd >= 0)

    def test_generic_station(self):
        s = Station('test')
        self.assertTrue(len(s.recording_frequencies),1)

    def test_station_frequencies(self):
        """ make sure the stations have between 1 and 3 frequencies """
        for _,stn in Station.get_all().items():
            n = len(stn.recording_frequencies)
            self.assertTrue(n>=1)
            self.assertTrue(n<=3)

    def test_station_weather(self):
        s = Station.from_name('HAY')
        data = s.get_weather('SEFD_info_230',2009,8,16)
        self.assertTrue(type(data['data']) is list)
        self.assertTrue(len(data['data']) > 0)
        self.assertTrue(type(data['data'][0]) is tuple)
        delete_sites()



"""
Copyright (c) 2022 Center for Astrophysics | Harvard & Smithsonian

This software is licensed for use as described in the LICENSE file in
the root directory of this distribution.

Testing Arrays

Originator: Aaron Oppenheimer March 2020
"""
from ngehtutil import *
import unittest

class TestClass(unittest.TestCase):
    def test_array_list(self):
        a = Array.get_list()
        self.assertEqual(type(a),list)
        self.assertTrue(len(a)>0)

    def test_default_array(self):
        d = Array.get_default_array_name()
        self.assertEqual(type(d),str)

        a = Array.get_default()
        self.assertEqual(type(a),Array)

    def test_array_init(self):
        with self.assertRaises(ValueError):
            a = Array('test','test')

        with self.assertRaises(ValueError):
            a = Array('test',['test'])



#
# object to describe a station
#
from pathlib import Path
import numpy as np
import pandas as pd
from ngehtutil.station_weather import get_weather_data

_THE_STATIONS = None


class Dish:
    """
    Class to describe a single dish antenna
    """
    diameter = 6 # diameter in meters
    surface_error = 0 # rms surface error in microns
    pointing_model = None # TBD description of pointing limits

    def __init__(self, diameter=6, surface_error=0, pointing_model=None):
        self.diameter = diameter
        self.surface_error = surface_error
        self.pointing_model = pointing_model

    def __str__(self):
        return f'{self.diameter}m dish'

    def __repr__(self):
        return f'{self.diameter}m dish'


class Station:
    id = None
    locality = None
    country = None
    latitude = None
    longitude = None
    elevation = None
    site_or_region = None

    owner = None
    register_converter = None
    polar_nonpolar = None
    existing_dish = None
    existing_infrastructure = None
    site_acquisition = None
    radiometer_testing = None
    uv_M87 = None
    uv_SgrA = None

    name = None
    dishes = None
    autonomy_of_operations = 'Manual'

    recording_bandwidth = 8
    recording_frequencies = []
    polarizations = 2
    sidebands = 2
    bit_depth = 2

    pwv = [0] * 12

    eht = False

    #
    # Static methods to get the list of possible items and to get the actual objects

    @staticmethod
    def get_list():
        """ Returns the list of stations we know about """
        return sorted(list(_THE_STATIONS.keys()))

    @staticmethod
    def from_name(name):
        """ Returns a station object from a name. """
        return _THE_STATIONS[name]

    @staticmethod
    def get_all():
        """ Returns all of the station objects we know about """
        return _THE_STATIONS

    def __init__(self, name, **kwargs):

        self.name = name

        # first see if there are dishes described
        count = kwargs.get('antenna_count',0)
        diameter = kwargs.get('dish_diameter',0)

        if diameter == 0:
            self.existing_dish = False
            self.dishes = None
        else:
            self.existing_dish = True
            if count==0:
                raise ValueError(f'Site {name} has no antennas')
            self.dishes = [Dish(diameter=diameter)] * count

        self.recording_frequencies = []
        for k in ['86', '230', '345']:
            n = kwargs.get(f'has_{k}',0) 
            if n == 1:
                self.recording_frequencies.append(k)
        if self.recording_frequencies == []:
            self.recording_frequencies = [230]

        for k,v in kwargs.items():
            # convert column headings in spreadsheet to our attribute names
            # todo - we skip some that might be helpful

            if k in ['antenna_count', 'dish_diameter', 'has_86', 'has_230', 'has_345']:
                pass
            else:
                setattr(self, k, v)

        # stn['lat'] = np.arcsin( stn['z']/np.sqrt(stn['x']**2+stn['y']**2+stn['z']**2) ) * 180./np.pi
        # stn['lon'] = np.arctan2( stn['y'], stn['x'] ) * 180.0/np.pi

    def __str__(self):
        return f'station {self.name}'

    def __repr__(self):
        return f'station {self.name}'

    def to_dict(self):
        ret = {}
        for k in self.__dir__():
            if not k[0:2] == '__':
                v = getattr(self, k)
                if isinstance(v,(str, float, int, list, dict, bool, np.integer)):
                    ret[k] = v
        return ret


    def set_diameter(self, diameter):
        """ sets the diameter of all of the dishes in the array. If there are no dishes
            defined, creates a single dish """
        if self.dishes:
            for d in self.dishes:
                d.diameter=diameter
        else:
            self.dishes=[Dish(diameter=diameter)]


    def data_rate(self):
        return self.recording_bandwidth * self.recording_frequencies * \
                        self.polarizations * self.sidebands * self.bit_depth * 2 # nyquist


    def xyz(self):
        """
        # convert latitude longitude and altitude into ECEF X Y Z coordinates
        # tewt tyoubg
        # x = ECEF X-coordinate (m)
        # y = ECEF Y-coordinate (m)
        # z = ECEF Z-coordinate (m)
        # lat = geodetic latitude (radians)
        # lon = longitude (radians)
        # alt = height above WGS84 ellipsoid (m)
        #
        % Notes: This function assumes the WGS84 model.
        %        Latitude is customary geodetic (not geocentric).
        %
        % Source: "Department of Defense World Geodetic System 1984"
        %         Page 4-4
        %         National Imagery and Mapping Agency
        %         Last updated June, 2004
        %         NIMA TR8350.2
        %
        % Michael Kleder, July 2005
        """
        # Convert Lat and Lon to radians
        lat = np.deg2rad(self.latitude)
        lon = np.deg2rad(self.longitude)
        alt = self.elevation
        # WGS84 ellipsoid constants:
        a = 6378137   
        e = 8.1819190842622e-2
        # intermediate calculation
        # (prime vertical radius of curvature)
        N = a / (1 - e**2 * np.sin(lat)**2)**0.5
        
        # results:
        x = (N+alt) * np.cos(lat) * np.cos(lon)
        y = (N+alt) * np.cos(lat)* np.sin(lon)
        z = ((1-e**2) * N + alt) * np.sin(lat)
        return [x, y, z]


    def SEFD(self, freq, elev, filled=0.7, month=5):
        """
        %
        % SEFD is the System Equivalent Flux Density
        %   freq is the measurement frequency (GHz)
        %   elev is the elevation angle of the telescope in degrees (0 to 90)

        %   filled is the geometric filling factor (unobscured telescope fraction)
        %   month is the observation month (1-12)

        %   Trx is the receiver temperature (K) - removed from arg list but could be back!

        % Get coefficients for PWV -> tau_0 based on frequency

        """
        if self.dishes is None:
            raise ValueError('Station needs a dish ')

        N = len(self.dishes)
        D = self.dishes[0].diameter
        RMS = self.dishes[0].surface_error
        PWV = self.pwv[month-1]

        if freq == 86:
            a = 0.0157; b = 0.00686
            Trx = 60 * 2/3
        elif freq == 230:
            a = 0.0107; b = 0.0431
            Trx = 136 * 2/3
        elif freq == 345:
            a = 0.0192; b = 0.152
            Trx = 219 * 2/3
        elif freq == 480:
            a = 0.123; b = 0.810
            Trx = 292 * 2/3
        elif freq == 690:
            a = 0.0664; b = 1.14
            Trx = 261 * 2/3
        else:
            print('Frequency fit for tau not known at this frequency')
            print('Available frequencies are 86, 225, 345, 480, 690 GHz')
            return 0

        # Calculate the aperture loss including Ruze loss due to roughness
        eta_A = filled * np.exp(-((4.0 * np.pi * RMS/3e5 * freq)**2))

        # DPFU is the "Degrees per Flux density Unit" which converts System
        # Temperature to flux density units
        DPFU = eta_A * N * (np.pi * (D/2)**2)/(2 * 1380)

        # Convert the Precipitable Water Vapor to Zenith Opacity (LB fits)
        tau_0 = a + b * PWV
        AM = 1/np.sin(np.deg2rad(elev))

        #Calculate the corrected System Temperature
        Tsys_star = np.exp(tau_0 * AM) * (Trx + (1 - np.exp(-tau_0 * AM)) * 290)

        # Calculate the SEFD
        the_SEFD = Tsys_star/DPFU
        return the_SEFD

    def diameter(self):
        """
        Return the effective diameter of the station
        """

        area = 0.0
        for dish in self.dishes:
            area += float(dish.diameter)**2.0

        return np.sqrt(area)


    def get_weather(self, type, year, month, day):
        """ returns weather info of the given type """

        return get_weather_data(self.name, type, year, month, day)

##
##
## Functions for bringing in the known sites
##
##


SITE_FILE_NAME = 'telescope_sites.xlsx'

def _site_file_path():
    path=str(Path(__file__).parent) + '/config'
    return f'{path}/{SITE_FILE_NAME}'

def _init_stations():
    """ do the initial setup on stations """
    global _THE_STATIONS
    if _THE_STATIONS is None:

        _THE_STATIONS = {}
        sites_data = pd.read_excel(_site_file_path(), index_col=0, \
            sheet_name='Basic Site Data', skiprows=[1])
        for s in sites_data.index:
            d = sites_data.loc[s]
            stn = Station(name=s, **dict(d))
            _THE_STATIONS[s] = stn

_init_stations()



import matplotlib as plt
import ehtim as eh
import sys


def main():
    plt.style.use('dark_background')
    for filepath in sys.argv[1:]:
        # extract file
        file = filepath.split('/')[-1]
        parts = file.split('.')
        if len(parts) == 1:
            fileroot = file
        else:
            fileroot = '.'.join(parts[0:-1])

        print(f'converting {fileroot}')
        im = eh.image.load_image(filepath)
        im.display(show=False, export_pdf=f"models/{fileroot}.pdf")

if __name__ == "__main__":
    main()


"""
Copyright (c) 2022 Center for Astrophysics | Harvard & Smithsonian

This software is licensed for use as described in the LICENSE file in
the root directory of this distribution.

Testing the Weather modeling code

Originator: Aaron Oppenheimer March 2020
"""
from multiprocessing.sharedctypes import Value
import ngehtutil.station_weather
from ngehtutil.station_weather import *
import unittest
from pathlib import Path


class TestClass(unittest.TestCase):
    def test_fetch_files(self):
        delete_sites()
        x = load_site('HAY',1)
        self.assertTrue(x > 1)
        homepath=str(Path(ngehtutil.station_weather.__file__).parent) + '/weather_data'
        self.assertTrue(os.path.isdir(f'{homepath}/HAY/01Jan'))
        self.assertTrue(os.path.exists(f'{homepath}/HAY/01Jan/RH.csv'))
        self.assertFalse(os.path.isdir(f'{homepath}/HAY/02Feb'))
        x = load_site('HAY',[1,2])
        self.assertTrue(x > 1)
        self.assertTrue(os.path.exists(f'{homepath}/HAY/02Feb/RH.csv'))
        
        # make sure that we don't reload files
        x = load_site('HAY',[1,2])
        self.assertTrue(x == 0)
        
        delete_sites()

    def test_delete_files(self):
        load_site('HAY')
        homepath=str(Path(ngehtutil.station_weather.__file__).parent) + '/weather_data'
        self.assertTrue(os.path.isdir(homepath))
        delete_sites()
        self.assertFalse(os.path.isdir(homepath))

    def test_fetch_fail(self):
        with self.assertRaises(ValueError):
            load_site('WOOHOO')
        with self.assertRaises(ValueError):
            load_site('HAY',15)
        with self.assertRaises(ValueError):
            load_site('HAY',[1,2,15])
        delete_sites()

    def test_get_data(self):
        data = get_weather_data('HAY','SEFD_info_230',2009,8,45)
        self.assertTrue(type(data) is dict)
        self.assertTrue(data['data'] is None)

        data = get_weather_data('HAY','SEFD_info_230',2009,8,16)
        self.assertTrue(type(data['data']) is list)
        self.assertTrue(len(data['data']) > 0)
        self.assertTrue(type(data['data'][0]) is tuple)

        data = get_weather_data('HAY','RH',2009,8,16)
        self.assertTrue(len(data['data']) > 0)
        self.assertTrue(type(data['data'][0]) is tuple)
        delete_sites()



"""
Copyright (c) 2022 Center for Astrophysics | Harvard & Smithsonian

This software is licensed for use as described in the LICENSE file in
the root directory of this distribution.

Testing the Source model

Originator: Aaron Oppenheimer March 2020
"""
from multiprocessing.sharedctypes import Value
from ngehtutil.source import Source
import unittest
import PIL


def get_default_source():
    return Source.get_source_list()[0]

class TestClass(unittest.TestCase):
    def test_source_list(self):
        a = Source.get_list()
        self.assertEqual(type(a),list)
        self.assertTrue(len(a)>0)


    def test_source_info(self):
        sl = Source.get_list()
        info = Source.from_name(sl[0])
        self.assertEqual(type(info),Source)

        with self.assertRaises(KeyError):
            info = Source.from_name('aaron')


    def test_source_description(self):
        sl = Source.get_list()
        s = Source.from_name(sl[0])
        desc = s.description
        self.assertEqual(type(desc),str)


    def test_source_image(self):
        sl = Source.get_list()
        s = Source.from_name(sl[0])
        im = s.picture(230)
        self.assertEqual(type(im), PIL.PngImagePlugin.PngImageFile)

        with self.assertRaises(ValueError):
            info = s.picture(999)


    def test_source_data_file(self):
        sl = Source.get_list()
        s = Source.from_name(sl[0])

        f = s.fits(230)
        self.assertEqual(type(f), str)

        with self.assertRaises(ValueError):
            f = s.fits(999)


    def test_default_source(self):
        s = Source.get_default()
        self.assertEqual(type(s),Source)


    def test_add_source(self):
        n1 = len(Source.get_list())
        attrs = {'230_data':'test1'}
        Source.add_source('testsource', **attrs)
        n2 = len(Source.get_list())
        self.assertTrue(n2 == n1 + 1)

        s = Source.from_name('testsource')
        self.assertTrue(s.name=='testsource')
        self.assertTrue(len(s.freq_list())==1)
        self.assertTrue(s.freq_list()[0]==230)
        self.assertTrue(s.fits(230)[-5:]=='test1')
        self.assertTrue(s.picture(230) is None)

        attrs = {'230_data':'/foo/test1'}
        Source.add_source('testsource2', **attrs)
        s2 = Source.from_name('testsource2')
        self.assertTrue(s2.fits(230)=='/foo/test1')

        Source.reinit_sources()
        n3 = len(Source.get_list())
        self.assertTrue(n1==n3)





"""
Copyright (c) 2022 Center for Astrophysics | Harvard & Smithsonian

This software is licensed for use as described in the LICENSE file in
the root directory of this distribution.

Manage things to do with sources

Originator: Aaron Oppenheimer March 2020
"""
from multiprocessing.sharedctypes import Value
from pathlib import Path
import pandas as pd
from PIL import Image
import os

_THE_SOURCES = None


class Source:

    name = None
    
    @staticmethod
    def get_list():
        return list(_THE_SOURCES.keys())

    @staticmethod
    def from_name(name):
        return _THE_SOURCES[name]

    @staticmethod
    def reinit_sources():
        global _THE_SOURCES
        _THE_SOURCES = None
        _init_sources()

    @staticmethod
    def add_source(name, **kwargs):
        global _THE_SOURCES
        _THE_SOURCES[name] = Source(name=name, **kwargs)

    @classmethod
    def get_default(cls):
        return cls.from_name(cls.get_list()[0])

    def __init__(self, name, **kwargs):
        self.name = name
        for k,v in kwargs.items():
            if v != '':
                setattr(self, k.lower(), v)

    def freq_list(self):
        the_list=[]
        for freq in [86, 230, 345, 480, 690]:
            if type(getattr(self,f'{freq}_data',None)) is str:
                the_list.append(freq)
        return the_list


    def picture(self, frequency):
        """ load the image file for a given source and frequency """

        if not frequency in self.freq_list():
            raise ValueError(f'No frequency {frequency} for source {self.name}')

        try:
            file_name = getattr(self,f'{int(frequency)}_image')
        except AttributeError:
            return None

        if file_name:
            path = os.path.abspath(__file__)
            dir_path = os.path.dirname(path)
            im = Image.open(f'{dir_path}/models/{file_name}')
            return im
        else:
            return None


    def fits(self, frequency):
        """ return the data file for a given source and frequency """
        if not frequency in self.freq_list():
            raise ValueError(f'No frequency {frequency} for source {self.name}')

        try:
            file_name = getattr(self, f'{int(frequency)}_data')
        except AttributeError:
            return None
            
        if file_name:
            if file_name[0] is not '/':
                path = os.path.abspath(__file__)
                dir_path = os.path.dirname(path)
                data = f'{dir_path}/models/{file_name}'
            else:
                data = file_name
            return data
        else:
            return None

    def __str__(self):
        return f'Source {self.name}'

    def __repr__(self):
        return f'Source {self.name}'


 ## Helper functions

def _init_sources():
    """ do the initial setup on sources """
    global _THE_SOURCES
    if _THE_SOURCES is None:
        _THE_SOURCES = {}
        path=str(Path(__file__).parent) + '/config'
        srcs = pd.read_csv(f'{path}/sources.csv', index_col=0)
        # _THE_SOURCES = {x:Source(name=x, **(srcs.loc[x].to_dict())) for x in srcs.index}
        for x in srcs.index:
            Source.add_source(name=x, **(srcs.loc[x].to_dict()))

_init_sources()

